@@ -1,7 +1,9 @@
 #!/bin/sh
 #
-# This script is used to compile SQLite and package everything up
-# so that it is ready to move to the SQLite website.
+# This script is used to compile SQLite and all its documentation and
+# ship everything up to the SQLite website.  This script will only work
+# on the system "zadok" at the Hwaci offices.  But others might find
+# the script useful as an example.
 #
 
 # Set srcdir to the name of the directory that contains the publish.sh
@@ -24,9 +26,7 @@ echo "VERSIONS: $VERS $VERSW"
 #
 make clean
 make sqlite3.c
-CFLAGS="-Os -DSQLITE_ENABLE_FTS3=1 -DSQLITE_THREADSAFE=0"
-echo '***** '"COMPILING sqlite3-$VERS.bin..."
-gcc $CFLAGS -Itsrc sqlite3.c tsrc/shell.c -o sqlite3 -ldl
+gcc -O2 -Itsrc sqlite3.c tsrc/shell.c -o sqlite3 -ldl -lpthread
 strip sqlite3
 mv sqlite3 sqlite3-$VERS.bin
 gzip sqlite3-$VERS.bin
@@ -35,31 +35,24 @@ mv sqlite3-$VERS.bin.gz doc
 
 # Build a source archive useful for windows.
 #
-make target_source
-cd tsrc
-echo '***** BUILDING preprocessed source archives'
-rm fts[12]* icu*
-rm -f ../doc/sqlite-source-$VERSW.zip
-zip ../doc/sqlite-source-$VERSW.zip *
-cd ..
-cp tsrc/sqlite3.h tsrc/sqlite3ext.h .
+make sqlite3.c
+cp tsrc/sqlite3.h .
 pwd
-zip doc/sqlite-amalgamation-$VERSW.zip sqlite3.c sqlite3.h sqlite3ext.h
+zip doc/sqlite-source-$VERSW.zip sqlite3.c sqlite3.h
 
 # Build the sqlite.so and tclsqlite.so shared libraries
 # under Linux
 #
+make sqlite3.c
 TCLDIR=/home/drh/tcltk/846/linux/846linux
 TCLSTUBLIB=$TCLDIR/libtclstub8.4g.a
-CFLAGS="-Os -DSQLITE_ENABLE_FTS3=1 -DHAVE_LOCALTIME_R=1 -DHAVE_GMTIME_R=1"
-echo '***** BUILDING shared libraries for linux'
-gcc $CFLAGS -shared tclsqlite3.c $TCLSTUBLIB -o tclsqlite3.so -lpthread
+gcc -O2 -shared -Itsrc sqlite3.c tsrc/tclsqlite.c $TCLSTUBLIB -o tclsqlite3.so
 strip tclsqlite3.so
 chmod 644 tclsqlite3.so
 mv tclsqlite3.so tclsqlite-$VERS.so
 gzip tclsqlite-$VERS.so
 mv tclsqlite-$VERS.so.gz doc
-gcc $CFLAGS -shared sqlite3.c -o sqlite3.so -lpthread
+gcc -O2 -shared -Itsrc sqlite3.c -o sqlite3.so
 strip sqlite3.so
 chmod 644 sqlite3.so
 mv sqlite3.so sqlite-$VERS.so
@@ -70,7 +63,6 @@ mv sqlite-$VERS.so.gz doc
 # Build the tclsqlite3.dll and sqlite3.dll shared libraries.
 #
 . $srcdir/mkdll.sh
-echo '***** PACKAGING shared libraries for windows'
 echo zip doc/tclsqlite-$VERSW.zip tclsqlite3.dll
 zip doc/tclsqlite-$VERSW.zip tclsqlite3.dll
 echo zip doc/sqlitedll-$VERSW.zip sqlite3.dll sqlite3.def
@@ -78,20 +70,19 @@ zip doc/sqlitedll-$VERSW.zip sqlite3.dll sqlite3.def
 
 # Build the sqlite.exe executable for windows.
 #
-OPTS='-DSTATIC_BUILD=1 -DNDEBUG=1 -DSQLITE_THREADSAFE=0'
-OPTS="$OPTS -DSQLITE_ENABLE_FTS3=1"
-i386-mingw32msvc-gcc -Os $OPTS -Itsrc -I$TCLDIR sqlite3.c tsrc/shell.c \
+make target_source
+OPTS='-DSTATIC_BUILD=1 -DNDEBUG=1'
+i386-mingw32msvc-gcc -O2 $OPTS -Itsrc -I$TCLDIR sqlite3.c tsrc/shell.c \
       -o sqlite3.exe
 zip doc/sqlite-$VERSW.zip sqlite3.exe
 
 # Construct a tarball of the source tree
 #
-echo '***** BUILDING source archive'
 ORIGIN=`pwd`
 cd $srcdir
 cd ..
 mv sqlite sqlite-$VERS
-EXCLUDE=`find sqlite-$VERS -print | egrep (CVS|www/|art/|doc/|contrib/) | sed 's,^, --exclude ,'`
+EXCLUDE=`find sqlite-$VERS -print | grep CVS | sed 's,^, --exclude ,'`
 tar czf $ORIGIN/doc/sqlite-$VERS.tar.gz $EXCLUDE sqlite-$VERS
 mv sqlite-$VERS sqlite
 cd $ORIGIN
@@ -127,6 +118,6 @@ mv $HOME/rpm/SRPMS/sqlite-$vers*.rpm doc
 # Build the website
 #
 #cp $srcdir/../historical/* doc
-#make doc
-#cd doc
-#chmod 644 *.gz
+make doc
+cd doc
+chmod 644 *.gz
